######## Fitting the model in the text #########################
######## User inputs ###########################################

nbl <- 20;                              # number of blocks
theta <- 1;					    #	estimate error variance ratio		
reps <- 100000;				    # bootstrap replication	

# read in data--ln yield estimates, two columns, removal and control for each block
y=read.table("example_data.txt")        # substitute your own data file except to run the example		

# print data 
print("",,FALSE)
print("The raw data--ln yield",,FALSE) 
print("Thinned  Control",,FALSE)
print(y) 


# calculate MLE estimates 
	
col.means <- apply(y,2,mean)   		# removal and control means 
resids <- t(sweep(t(y),c(1,2),col.means))	# Sweep out treatment means
SSP <- cov(y,y)					# sample variance and covariance estimates

###### a' (ap), a, b'(bp), b estimates 

bp <- (SSP[2,2] - theta*SSP[1,1] + sqrt((theta*SSP[1,1]-SSP[2,2])^2 + 4*theta*SSP[1,2]^2))/2/SSP[1,2];

bhat <- 1 - bp;			# estimated slope of the competition-environment relationship

print("",,FALSE)
print("MLE of the intercept of the competition-environment relationship, b",,FALSE)
print(bhat);

Ebar <- col.means[1];		# mean environmental response

ap <- col.means[2]- Ebar*bp; 

ahat <- -ap;			# intercept of the competition-environment relationship

print("",,FALSE)
print("MLE of the intercept of the competition-environment relationship, a",,FALSE)
print(ahat);

###### Estimates of E

Etilde_1 <- (theta*resids[,1] + bp*resids[,2])/(theta + bp^2);

Etilde_0 <- (theta*resids[,1] + resids[,2])/(theta+1);

E_1 = Etilde_1 + Ebar;		# Estimated environmental responses with the general hypothesis of covariance
E_0 = Etilde_0 + Ebar;		# Estimated environmental responses under the null hypothes of no covariance

## Now the sum of squares 

SSE_1 <- sum((resids[,1] - Etilde_1)^2 + (resids[,2] - Etilde_1*bp)^2/theta); 

SSE_0 <- sum((resids[,1] - Etilde_0)^2 + (resids[,2] - Etilde_0)^2/theta);

print("",,FALSE)
print("Error sum of squares for the model",,FALSE) 
print(SSE_1);

print("",,FALSE)
print("Error sum of squares for the null",,FALSE)
print(SSE_0);

##### Do the F-test 


F <- (SSE_0 - SSE_1)*(nbl-2)/SSE_1;           # Calculate F

print("",,FALSE)
print("F statistic",,FALSE);
print(F);

P_samp <- pf(F,1,nbl-2,0,FALSE,FALSE);	    # Significance of that F

print("",,FALSE)
print("uncorrected P value",,FALSE)
print(P_samp);

##### Calculate Variances and Covariances 

Var_E_est <- SSP[1,2]/bp;                      # Variance estimate of E

print("",,FALSE)
print("Estimated Variance of Environmental Response",,FALSE)
print(Var_E_est)

Var_C_est <- Var_E_est*bhat^2;                 # Corresponding estimate variance of variance of C 

print("",,FALSE)
print("Estimated Variance of Competitive Response",,FALSE)
print(Var_C_est)


Cov_EC_est <- Var_E_est*bhat;                  # Estimated covariance between environment and competition 
print("",,FALSE)
print("Estimated Covariance between Environment and Competition",,FALSE)
print(Cov_EC_est)

Var_eps_R_est <- SSP[1,1] - Var_E_est;         # Estimated removal error variance
print("",,FALSE)
print("Estimated error variance for the removal",,FALSE)
print(Var_eps_R_est)

Var_eps_N_est <- theta*Var_eps_R_est;          # Estimated nonremoval error variance given theta = 1
print("",,FALSE)
print("Estimated error variance for the control",,FALSE)
print(Var_eps_N_est)

#### Back transformed estimates 

Est_mean_Y_T_wo_var <- exp(Ebar);              # back transformed mean from log scale 
print("",,FALSE)
print("Back transformed mean of removal",,FALSE)
print(Est_mean_Y_T_wo_var)


Est_mean_Y_T = exp(Ebar)*exp(Var_E_est/2);     # Back transformed mean accounting for environmental variation @ Note, this assumes sampling error is not actual plant variation, 
print("",,FALSE)
print("Back transformed mean of removal, accounting for environmental variance",,FALSE)
print(Est_mean_Y_T)
                                               
Est_mean_Y_U_wo_var <- exp(col.means[2]);      # back transformed estimate of mean from logscale @;
print("",,FALSE)
print("Back transformed mean of control",,FALSE)
print(Est_mean_Y_U_wo_var)


Est_mean_Y_U_wo_cov = exp(col.means[2])*exp(Var_E_est/2 + Var_C_est/2);
                                               # back transformed mean with environmental and competitive variance, but not covarincewithout the covariance 
print("",,FALSE)
print("Back transformed mean of control accounting for environmental and competitive variance",,FALSE)
print(Est_mean_Y_U_wo_cov)

                    
Est_mean_Y_U = Est_mean_Y_U_wo_cov*exp(-Cov_EC_est); # back transformed mean with the covariance estimate of the total think
print("",,FALSE)
print("Back transformed mean of control: the full model with covariance",,FALSE)
print(Est_mean_Y_U)


#### Model fits

print("",,FALSE)

Ehat <- E_1;						# Estimated environmental response in each block
print("",,FALSE)
print("Estimated environmental response for each block",,FALSE)
print(Ehat);

Chat <- ahat + bhat*Ehat;				# Estimated competition in each block
print("",,FALSE)
print("Estimated competitive response for each block",,FALSE)
print(Chat)

Mean_ln_Y <- Ehat - Chat;				# mean log yield under the model
print("",,FALSE)
print("Mean log yield: E-C estimated under the model for each plot",,FALSE)
print(Mean_ln_Y)

#### Bootstrap correction of the P-value


#**************************** parameters of the runs ********************************

s2E <- Var_E_est;       # Variance of E              
s2ER <- Var_eps_R_est;  # Removal Error variance    
s2NR <- Var_eps_N_est;  # NonRemoval Error Variance 
theta_hat <- s2NR/s2ER; # Error variance ratio--same as theta provided in this instance      

nb <- nbl;              # Number of blocks (E values) in the experiment 

sE = sqrt(s2E);         # Convert variances to standard deviations 
sER = sqrt(s2ER);
sNR = sqrt(s2NR);


#***************************** The runs *********************************************/


cumP_mle <- rep(0,reps);	# Set up vector to receive P values

for (i in 1:reps) { 
				
	E <- rnorm(nb,Ebar,sE);
	C <- rep(0,nb);		
	
	y_b <- cbind(E+rnorm(nb,0,sER),E-C+rnorm(nb,0,sNR));

	# calculate MLE estimates 
	
	col.means_b <- apply(y_b,2,mean) 	  		# removal and control means 
	resids_b <- t(sweep(t(y_b),c(1,2),col.means_b))	# Sweep out treatment means
	SSP_b <- cov(y_b,y_b);					# sample variance and covariance estimates

	######  bootstrap estimates; 

	bp_b <- (SSP_b[2,2] - theta_hat*SSP_b[1,1] + sqrt((theta_hat*SSP_b[1,1]-SSP_b[2,2])^2 + 4*theta_hat*SSP_b[1,2]^2))/2/SSP_b[1,2];

	bhat_b <- 1 - bp_b;			# estimated slope of the competition-environment relationship

	Ebar_b <- col.means_b[1];		# mean environmental response

	ap_b <- col.means_b[2]- Ebar_b*bp_b; 

	ahat_b <- -ap_b;			# intercept of the competition-environment relationship

	###### Estimates of E

	Etilde_1_b <- (theta_hat*resids_b[,1] + bp_b*resids_b[,2])/(theta_hat + bp_b^2);

	Etilde_0_b <- (theta_hat*resids_b[,1] + resids_b[,2])/(theta_hat+1);

	## Now the sum of squares 

	SSE_1_b <- sum((resids_b[,1] - Etilde_1_b)^2 + (resids_b[,2] - Etilde_1_b*bp_b)^2/theta_hat); 

	SSE_0_b <- sum((resids_b[,1] - Etilde_0_b)^2 + (resids_b[,2] - Etilde_0_b)^2/theta_hat);


	##### Do the F-test 

	F_b <- (SSE_0_b - SSE_1_b)*(nb-2)/SSE_1_b;    # Calculate F

	P_boot <- pf(F_b,1,nb-2,0,FALSE,FALSE);	    # Significance of that F

	cumP_mle[i] <- P_boot;}


#### Now get correct the sample bootstrap with the sample data

 
P_boot = sum(cumP_mle <= P_samp)/reps;

print("",,FALSE)
print("Bootstrap p value for the data",,FALSE); 
print(P_boot);
